@@ -40,6 +40,8 @@ from .monitor import (
     check_interval
 )
 
+_recover_interaction = False
+
 
 def _add_bls_key_to_validator():
     """
@@ -59,12 +61,23 @@ def _add_bls_key_to_validator():
 def _send_edit_validator_tx(bls_key_to_add):
     log(f"{Typgpy.OKBLUE}Adding bls key: {Typgpy.OKGREEN}{bls_key_to_add}{Typgpy.OKBLUE} "
         f"to validator: {Typgpy.OKGREEN}{validator_config['validator-addr']}{Typgpy.ENDC}")
-    response = cli.single_call(f"hmy --node={node_config['endpoint']} staking edit-validator "
-                               f"--validator-addr {validator_config['validator-addr']} "
-                               f"--add-bls-key {bls_key_to_add} --passphrase-file {saved_wallet_pass_path} "
-                               f"--bls-pubkeys-dir {bls_key_dir}")
-    log(f"{Typgpy.OKBLUE}Edit-validator transaction response: "
-        f"{Typgpy.OKGREEN}{response}{Typgpy.ENDC}")
+    count = 0
+    while True:
+        count += 1
+        try:
+            response = cli.single_call(f"hmy --node={node_config['endpoint']} staking edit-validator "
+                                       f"--validator-addr {validator_config['validator-addr']} "
+                                       f"--add-bls-key {bls_key_to_add} --passphrase-file {saved_wallet_pass_path} "
+                                       f"--bls-pubkeys-dir {bls_key_dir}")
+            log(f"{Typgpy.OKBLUE}Edit-validator transaction response: "
+                f"{Typgpy.OKGREEN}{response}{Typgpy.ENDC}")
+            return
+        except (RuntimeError, TimeoutError, ConnectionError, subprocess.CalledProcessError) as e:
+            log(f"{Typgpy.FAIL}Edit-validator transaction failure (attempt {count}). Error: {e}{Typgpy.ENDC}")
+            if not _recover_interaction:
+                raise e
+            log(f"{Typgpy.WARNING}Trying again in {check_interval} seconds.{Typgpy.ENDC}")
+            time.sleep(check_interval)
 
 
 def _create_new_validator():
@@ -83,25 +96,28 @@ def _verify_staking_epoch():
     while curr_epoch < staking_epoch:  # WARNING: using staking epoch for extra security of configs.
         sys.stdout.write(f"\rWaiting for staking epoch ({staking_epoch}) -- current epoch: {curr_epoch}")
         sys.stdout.flush()
-        time.sleep(8)  # Assumption of 8 second block time...
+        time.sleep(check_interval)
         curr_epoch = get_current_epoch(node_config['endpoint'])
     log(f"{Typgpy.OKGREEN}Network is at or past staking epoch{Typgpy.ENDC}")
 
 
 def _verify_account_balance(amount):
-    """
-    Invariant: Account has sufficient balance before sending transactions
-    """
+    count = 0
     log(f"{Typgpy.OKBLUE}Verifying Balance...{Typgpy.ENDC}")
-    if not check_min_bal_on_s0(validator_config['validator-addr'], amount, node_config['endpoint']):
-        log(f"{Typgpy.FAIL}Cannot create validator, {validator_config['validator-addr']} "
-            f"does not have sufficient funds ({amount} ONE).{Typgpy.ENDC}")
-        raise SystemExit("Create Validator Error")
-    else:
-        log(f"{Typgpy.OKGREEN}Address: {validator_config['validator-addr']} has enough funds{Typgpy.ENDC}")
+    while True:
+        count += 1
+        if not check_min_bal_on_s0(validator_config['validator-addr'], amount, node_config['endpoint']):
+            log(f"{Typgpy.FAIL}Cannot create validator, {validator_config['validator-addr']} "
+                f"does not have sufficient funds (need {amount} ONE). Checked {count} times.{Typgpy.ENDC}")
+            if not _recover_interaction:
+                raise SystemExit("Create Validator Error")
+            log(f"{Typgpy.WARNING}Checking again in {check_interval} seconds.{Typgpy.ENDC}")
+            time.sleep(check_interval)
+        else:
+            log(f"{Typgpy.OKGREEN}Address: {validator_config['validator-addr']} has enough funds{Typgpy.ENDC}")
+            return
 
 
-# TODO: move this to node.py and update libs accordingly
 def verify_node_sync():
     log(f"{Typgpy.OKBLUE}Verifying Node Sync...{Typgpy.ENDC}")
     wait_for_node_response("http://localhost:9500/", sleep=1, verbose=True)
@@ -115,7 +131,7 @@ def verify_node_sync():
         log(f"{Typgpy.OKBLUE}Deactivating validator until node is synced.{Typgpy.ENDC}")
         try:
             deactivate_validator()
-        except (TimeoutError, RuntimeError, subprocess.CalledProcessError) as e:
+        except (TimeoutError, ConnectionError, RuntimeError, subprocess.CalledProcessError) as e:
             log(f"{Typgpy.FAIL}Unable to deactivate validator {validator_config['validator-addr']}"
                 f"error {e}. Continuing...{Typgpy.ENDC}")
     while curr_epoch_shard < ref_epoch or curr_epoch_beacon < ref_epoch:
@@ -141,39 +157,52 @@ def verify_node_sync():
         time.sleep(check_interval)  # Wait for nonce to finalize before sending activate
     try:
         activate_validator()
-    except (TimeoutError, RuntimeError, subprocess.CalledProcessError) as e:
+    except (TimeoutError, ConnectionError, RuntimeError, subprocess.CalledProcessError) as e:
         log(f"{Typgpy.FAIL}Unable to activate validator {validator_config['validator-addr']}"
             f"error {e}. Continuing...{Typgpy.ENDC}")
 
 
 def _send_create_validator_tx():
     log(f"{Typgpy.OKBLUE}Sending create validator transaction...{Typgpy.ENDC}")
-    response = cli.single_call(f'hmy --node={node_config["endpoint"]} staking create-validator '
-                               f'--validator-addr {validator_config["validator-addr"]} '
-                               f'--name "{validator_config["name"]}" '
-                               f'--identity "{validator_config["identity"]}" '
-                               f'--website "{validator_config["website"]}" '
-                               f'--security-contact "{validator_config["security-contact"]}" '
-                               f'--details "{validator_config["details"]}" '
-                               f'--rate {validator_config["rate"]} '
-                               f'--max-rate {validator_config["max-rate"]} '
-                               f'--max-change-rate {validator_config["max-change-rate"]} '
-                               f'--min-self-delegation {validator_config["min-self-delegation"]} '
-                               f'--max-total-delegation {validator_config["max-total-delegation"]} '
-                               f'--amount {validator_config["amount"]} '
-                               f'--bls-pubkeys {",".join(node_config["public-bls-keys"])} '
-                               f'--passphrase-file "{saved_wallet_pass_path}" '
-                               f'--bls-pubkeys-dir "{bls_key_dir}" ')
-    log(f"{Typgpy.OKBLUE}Created Validator!{Typgpy.OKGREEN}{response}{Typgpy.ENDC}")
+    count = 0
+    while True:
+        count += 1
+        try:
+            response = cli.single_call(f'hmy --node={node_config["endpoint"]} staking create-validator '
+                                       f'--validator-addr {validator_config["validator-addr"]} '
+                                       f'--name "{validator_config["name"]}" '
+                                       f'--identity "{validator_config["identity"]}" '
+                                       f'--website "{validator_config["website"]}" '
+                                       f'--security-contact "{validator_config["security-contact"]}" '
+                                       f'--details "{validator_config["details"]}" '
+                                       f'--rate {validator_config["rate"]} '
+                                       f'--max-rate {validator_config["max-rate"]} '
+                                       f'--max-change-rate {validator_config["max-change-rate"]} '
+                                       f'--min-self-delegation {validator_config["min-self-delegation"]} '
+                                       f'--max-total-delegation {validator_config["max-total-delegation"]} '
+                                       f'--amount {validator_config["amount"]} '
+                                       f'--bls-pubkeys {",".join(node_config["public-bls-keys"])} '
+                                       f'--passphrase-file "{saved_wallet_pass_path}" '
+                                       f'--bls-pubkeys-dir "{bls_key_dir}" ')
+            log(f"{Typgpy.OKBLUE}Create-validator transaction response: "
+                f"{Typgpy.OKGREEN}{response}{Typgpy.ENDC}")
+            return
+        except (RuntimeError, TimeoutError, ConnectionError, subprocess.CalledProcessError) as e:
+            log(f"{Typgpy.FAIL}Create-validator transaction failure (attempt {count}). Error: {e}{Typgpy.ENDC}")
+            if not _recover_interaction:
+                raise e
+            log(f"{Typgpy.WARNING}Trying again in {check_interval} seconds.{Typgpy.ENDC}")
+            time.sleep(check_interval)
 
 
 def setup(recover_interaction=False):
+    global _recover_interaction
+    _recover_interaction = recover_interaction
     old_logging_handlers = logging.getLogger('AutoNode').handlers.copy()
     logging.getLogger('AutoNode').addHandler(get_simple_rotating_log_handler(log_path))
     log(f"{Typgpy.HEADER}Starting validator setup...{Typgpy.ENDC}")
     if node_config['no-validator']:
         raise SystemExit(f"{Typgpy.WARNING}Node config specifies not validator automation, exiting...{Typgpy.ENDC}")
-    auto_interaction = 'Y' if recover_interaction else None
 
     log(f"{Typgpy.OKBLUE}Create validator config: {Typgpy.OKGREEN}"
         f"{json.dumps(validator_config, indent=4)}{Typgpy.ENDC}")
@@ -185,12 +214,12 @@ def setup(recover_interaction=False):
         if validator_config['validator-addr'] in all_val_address:
             log(f"{Typgpy.WARNING}{validator_config['validator-addr']} already in list of validators!{Typgpy.ENDC}")
             prompt = "Add BLS key(s) to existing validator? [Y]/n \n> "
-            if input_with_print(prompt, auto_interaction) in {'Y', 'y', 'yes', 'Yes'}:
+            if input_with_print(prompt, 'Y' if recover_interaction else None) in {'Y', 'y', 'yes', 'Yes'}:
                 log(f"{Typgpy.HEADER}{Typgpy.BOLD}Editing validator...{Typgpy.ENDC}")
                 _add_bls_key_to_validator()
         elif validator_config['validator-addr'] not in all_val_address:
             prompt = "Create validator? [Y]/n \n> "
-            if input_with_print(prompt, auto_interaction) in {'Y', 'y', 'yes', 'Yes'}:
+            if input_with_print(prompt, 'Y' if recover_interaction else None) in {'Y', 'y', 'yes', 'Yes'}:
                 log(f"{Typgpy.HEADER}{Typgpy.BOLD}Creating new validator...{Typgpy.ENDC}")
                 _create_new_validator()
         else:
